# -*- coding: utf-8 -*-
import json
import os
import pickle
import re
from collections import defaultdict
from pathlib import Path

from tqdm import tqdm


def remove_spaces(text):
    # remove begin/ending spaces
    text = text.strip()
    # replace \t with regular space
    text = re.sub("\t", " ", text)
    # remove consecutive spaces
    text = re.sub(" +", " ", text)
    return text


class ATRDatasetFormatter:
    """
    Global pipeline/functions for dataset formatting
    """

    def __init__(self, dataset: Path, image_format: str, remove_spaces: bool):
        self.dataset = dataset
        self.set_names = ["train", "val", "test"]
        self.remove_spaces = remove_spaces

        self.image_folder = self.dataset / "images"
        self.labels_folder = self.dataset / "labels"

        self.image_format = image_format
        if self.image_format.startswith("."):
            self.image_format = self.image_format[1:]

    def format(self):
        """
        Format ATR dataset
        """
        ground_truth = defaultdict(dict)
        charset = set()
        for set_name in self.set_names:
            set_folder = self.labels_folder / set_name
            for file_name in tqdm(
                os.listdir(set_folder), desc="Formatting " + set_name
            ):
                data = self.parse_labels(set_name, file_name)
                charset = charset.union(set(data["label"]))
                ground_truth[set_name][data["img_path"]] = {
                    "text": data["label"],
                }
        return ground_truth, charset

    def read_file(self, file_name):
        with open(file_name, "r") as f:
            text = f.read()
            if self.remove_spaces:
                text = remove_spaces(text)
            return text.strip()

    def parse_labels(self, set_name, file_name):
        return {
            "img_path": os.path.join(
                self.image_folder,
                set_name,
                f"{os.path.splitext(file_name)[0]}.{self.image_format}",
            ),
            "label": self.read_file(
                os.path.join(self.labels_folder, set_name, file_name)
            ),
        }

    def run(self):
        ground_truth, charset = self.format()

        with open(self.dataset / "labels.json", "w") as f:
            json.dump(
                ground_truth,
                f,
                sort_keys=True,
                indent=4,
            )
        with open(self.dataset / "charset.pkl", "wb") as f:
            pickle.dump(sorted(list(charset)), f)


def run(dataset, image_format, keep_spaces):
    ATRDatasetFormatter(
        dataset=dataset, image_format=image_format, remove_spaces=not keep_spaces
    ).run()
